from __future__ import print_function
import os

_GENERICIO_NO_MPI = False
if "GENERICIO_NO_MPI" in os.environ:
    _GENERICIO_NO_MPI = os.environ["GENERICIO_NO_MPI"].lower() in ["true", "yes", "y"]

if _GENERICIO_NO_MPI:
    print(
        "WARNING: the pygio module without MPI support has been loaded (due to the GENERICIO_NO_MPI env variable). Writing GenericIO files not supported."
    )
    from .pygio_nompi import *

else:
    # try to load the MPI library (or the no-mpi library, in case of missing MPI during compilation)
    from . import pygio as _pygio

    try:
        _pygio._init_mpi()
    except:
        print(
            "WARNING: The pygio module has been compiled without MPI support. Writing GenericIO files not supported."
        )
        _GENERICIO_NO_MPI = True
    from .pygio import *


# move some ENUMs and static functions up to the module namespace
FileIO = PyGenericIO.FileIO
MismatchBehavior = PyGenericIO.MismatchBehavior
setDefaultShouldCompress = PyGenericIO.setDefaultShouldCompress
setNaturalDefaultPartition = PyGenericIO.setNaturalDefaultPartition
setDefaultFileIOType = PyGenericIO.setDefaultFileIOType
if not _GENERICIO_NO_MPI:
    setCollectiveMPIIOThreshold = PyGenericIO.setCollectiveMPIIOThreshold

# sensible defaults?
if not _GENERICIO_NO_MPI:
    setNaturalDefaultPartition()
    setDefaultFileIOType(FileIO.FileIOMPICollective)
